/**
 * request 网络请求工具
 * 更详细的 api 文档: https://github.com/umijs/umi-request
 */
import { extend } from 'umi-request'
import { message, notification } from 'antd'
import { getTokenid, getUserAccount, redirectToLogin, saveTokenId } from './common'
import consts from '@/consts'

const RETRY_TOKEN_TIME = 3 // 重新获取token次数
let currentTokenRetry = RETRY_TOKEN_TIME // 当前token刷新次数
const codeMessage = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。'
}
/**
 * 异常处理程序
 */

const errorHandler = error => {
  const { response } = error

  if (response && response.status) {
    const errorText = codeMessage[response.status] || response.statusText
    const { status, url } = response
    notification.error({
      message: `请求错误 ${status}: ${url}`,
      description: errorText
    })
  } else if (!response) {
    notification.error({
      description: '您的网络发生异常，无法连接服务器',
      message: '网络异常'
    })
  }

  return response
}


/**
 * 配置request请求时的默认参数
 */

const request = extend({
  errorHandler,
  // 默认错误处理
  credentials: 'include' // 默认请求是否带上cookie
})


/**
 * 获取token令牌
 */
async function apiGetToken() {
  const data = await request.get(consts.TOKEN_API, {})
  return data
}

// 请求拦截器
request.interceptors.request.use((url, options) => {
  const token = getTokenid()
  if (token) {
    // 在白名单里的请求放过
    if (consts.TOKEN_WHITE_LIST.includes(url)) {
      return { url, options }
    }
    // eslint-disable-next-line no-param-reassign
    options.headers[consts.TOKEN_KEY] = `bearer ${token}`
    return { url, options }
  }
  return { url, options }
})
const refreshTokenIfNeed = async (response, config) => {
  const res = await response.clone().json()
  if (
    res &&
    res.code !== consts.RET_CODE.SUCCESS &&
    !consts.TOKEN_INVALID_CODE.includes(res.code)
  ) {
    message.error((res && res.msg) || '请求错误')
    return response
  }

  if (
    res &&
    res.code &&
    consts.TOKEN_INVALID_CODE.includes(res.code)
  ) {
    const account = getUserAccount()
    if (!account || !currentTokenRetry) {
      redirectToLogin()
      return response
    }
    currentTokenRetry -= 1
    // token过期或者没有传token
    const data = await apiGetToken(account)
    if (data && data.data && data.data.code === consts.RET_CODE.SUCCESS) {
      currentTokenRetry = RETRY_TOKEN_TIME // 重置token重试次数
      // 刷新token成功，保存token
      const token = data.data && data.data.token
      saveTokenId(token)
      // 重新发起请求
      const newResponse = await request(response.url, config)
      return newResponse
    }
  }
  return response
}

// 响应拦截器
request.interceptors.response.use((response, options) => {
  return refreshTokenIfNeed(response, options)
})

export default request
